/******************************************************************************
 *  loop.c -- Loopback test program for the S5920 prototype board             *
 *                                                                            *
 *  (c) 1998 Greg Burley                                                      *
 *                                                                            *
 *                                                                            *
 *  This program tests the S5920 prototype board using the standard           *
 *  read()/write()/ioctl() functions.                                         *
 *                                                                            *
 *  It uses the real driver ("/dev/amcc") and the PCI interface.              *
 *                                                                            *
 ******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
// #include <sys/time.h>
#include <asm/io.h>                 /* for inl() and outl() */
#include <asm/types.h>
#include "amcc.h"

#define  NBLOCKS        32

#define  BSIZE          1024
#define  FIFO_SIZE      16384

static int  amcc = 0L;           /* device */

int clear_fifo(void);
void printrow(ushort buf[],int start, int finish);
void msleep(int msec);

/*========== Main program ====================================================*/
int
main ()
{
    u_int icsr, rcr, ptcr, mbef;
    u_int wb, rb;
    int i,j,r,w,x,z;
    int val=0;
    int rc=0, wc=0, ec=0, err=0;
    int hist[16];
    u_short wbuf[BSIZE], rbuf[BSIZE];
    u_char cmd=0;

    /* open the device */
    amcc = open ("/dev/amcc0", O_RDWR);
    if (amcc == -1){
        printf ("Error opening device /dev/amcc0 \n");
        exit(0);
    }
    else {
        printf ("Device /dev/amcc0 opened\n");
    }


    /* Reset + scan the operation registers */
    r = ioctl(amcc, AMCC_RESET_ADDON);
    r = ioctl(amcc, AMCC_GET_ICSR, &icsr);
    r = ioctl(amcc, AMCC_GET_RCR , &rcr);
    r = ioctl(amcc, AMCC_GET_PTCR, &ptcr);
    r = ioctl(amcc, AMCC_GET_MBEF, &mbef);
    printf("ICSR %08x  MBEF %08x  RCR %08x  PTCR %08x\n",
            icsr, mbef, rcr, ptcr);

    clear_fifo();
    srand(time(NULL));

    x = 0;
    while (cmd != 'q'){
        fprintf(stdout,"\n");
        fprintf(stdout,"cmd>");
        fscanf(stdin,"%s",&cmd);

        switch(cmd){
        case 's':
        r = clear_fifo();
        wbuf[0] = (x>>0 & 0xffff);
        r = write(amcc,wbuf,2);
        r = read(amcc,rbuf,2);
        val = (int)(rbuf[0] & 0xffff);
        printf("send=%04x  get=%06x \n",x,val);
        x++;
        break;

        case 'v':
        r = clear_fifo();
        break;

        case 'b':           /* write, then read a small buffer */
        r = clear_fifo();
        w=0;
        r=0;
        err=0;
        for (r=0; r<BSIZE; r++){
            wbuf[r] = ((r & 0xff)<<0)|((r & 0xff)<<8);
            rbuf[r] = 0;
        }
        w = write(amcc,wbuf,2*BSIZE);
        sleep(1);
        r = read(amcc,rbuf,2*BSIZE);

        /* Print the read and write buffers (should be identical) */
        printf("\n");
        for (j=0; j<BSIZE; j+=2){
            wb = *((u_int *)(&wbuf[j]));
            rb = *((u_int *)(&rbuf[j]));
            if (j<32)printf("wbuf %08x  rbuf %08x\n",wb, rb);
        }

        /* Check to see if read and write buffers are identical */
        for (j=0; j<BSIZE; j++){
            if (wbuf[j] != rbuf[j]) err++;
        }
        printf("w %04d  r %04d  errors %d\n",w,r,err);

        /* Make a histogram of bits received */
        for (j=0; j<16; j++) hist[j] = 0;
        for (i=0; i<BSIZE; i++){
            for (j=0; j<16; j++){
                if (rbuf[i] & (0x1<<j)) hist[j]++;
            }
        }

        for (j=0; j<16; j++){
            printf("%4d ",hist[j]);
            if (j==7 || j==15)printf("\n");
        }
        break;

        case 'm':
        ec = 0;
        wc = 0;
        rc = 0;
        clear_fifo();
        for (i=0; i<BSIZE; i++){
            z = (int)(rand()& 0xffff);
            wbuf[i] = ((z & 0xff)<<0)|((z & 0xff)<<8);
        }
        for (j=0; j<NBLOCKS; j++){
            for (i=0; i<BSIZE; i++) rbuf[i] = 0;
            w = write(amcc,wbuf,2*BSIZE);
            r = read(amcc,rbuf,2*BSIZE);
            wc += w;
            rc += r;
            err = 0;
            for (i=0; i<BSIZE; i++){
                if (wbuf[i] != rbuf[i]) err++;
            }
            ec += err;
            if (err>0)break;

        }
        for (j=0; j<BSIZE; j+=2){
            wb = *((u_int *)(&wbuf[j]));
            rb = *((u_int *)(&rbuf[j]));
            if (j<64)printf("wbuf %08x  rbuf %08x\n",wb, rb);
        }
        printf("w %04d  r %04d  err %d\n",wc,rc,ec);
        break;

        default:
            break;
        }
    }


    /* close down */
    if (amcc){
        close (amcc);
        printf("Device /dev/amcc0 closed\n");
    }

    return EXIT_SUCCESS;    /* to make gcc happy */
}


int
clear_fifo(void)
{
    extern int amcc;
    int count=0;

    (void)ioctl(amcc, AMCC_RESET_FIFO, &count);
    if (count > 0)printf("Clearing FIFO  count=  %d\n",count);
    return(count);
}

void
printrow(ushort buf[],int start, int finish)
{
    int j;

    for(j=start; j<finish; j++) printf("%04x ",buf[j]);
    printf("\n");
}

